<?php
/*
Plugin Name: BuddyPress Community Activate
Plugin URI: http://davidbisset.com/code/buddypress/invite-anyone/
Description: Allows already-registered BuddyPress members to choose who else gets access to the BuddyPress site, turning a BuddyPress site into more of an "exclusive club". Kinda inspired by communities like Forrst.
Version: 0.2.0
Author: David Bisset
Author URI: http://davidbisset.com
*/

define ( 'BP_COMMUNITY_ACTIVATE_VERSION', '0.2.0' );
define ( 'BP_COMMUNITY_ACTIVATE_DBVERSION', 0 );

/**
 * Invite Limit - This will eventually be an option controllable in a WordPress admin panel
 */
define ( 'CACTIVATE_INVITE_LIMIT', '2' );

// You can override these in wp-config.php.
if ( !defined( 'CACTIVATE_SLUG' ) )
        define ( 'CACTIVATE_SLUG', __( 'invites', 'bpca' ) );
        
        
/**
 * Only load the component if BuddyPress is loaded and initialised. 
 *
 * @since 0.1
 */
function bpca_init() {
        require( dirname( __FILE__ ) . '/includes/bp-community-activate-core.php' );
	if ( is_admin() )
		require( dirname( __FILE__ ) . '/includes/bp-community-activate-admin.php' );
        do_action( 'bpca_init' );
}
add_action( 'bp_include', 'bpca_init' );

/**
 * This sets some initial values when the plugin is activated. 
 *
 * @since 0.2
 */
function bpca_activation() {
	if ( !$bpcaoptions = get_option( 'bp_community_activate' ) )
		$bpcaoptions = array();

	if ( !$bpcaoptions['max_activations'] )
		$bpcaoptions['max_activations'] = 5;

	$bpcaoptions['version'] = BP_COMMUNITY_ACTIVATE_VERSION;

	update_option( 'bp_community_activate', $bpcaoptions );
}
register_activation_hook( __FILE__, 'bpca_activation' );

/**
 * this prevents a standard activation email from going out
 *
 * @since 0.1
 */
function bpca_disable_activation_email() {
    return false;
}
add_filter('bp_core_signup_send_activation_key', 'bpca_disable_activation_email');

/**
 * Send email which notifies users they are waiting to be activated
 *
 * @since 0.1
 */
function bpca_send_in_queue_email( $user_id ) {
    global $bp, $wpdb;

    $user = new WP_User($user_id);

    $user_login = stripslashes($user->user_login);
    $user_email = stripslashes($user->user_email);
    $activation_key = wp_hash( $user_id );
    update_user_meta( $user_id, 'activation_key', $activation_key );
    $activate_url = bp_get_activation_page() ."?key=$activation_key";
    $activate_url = esc_url( $activate_url );

    $message  = sprintf(__('Thank you for registering. Click the below link to confirm you made this request, and then you will be placed into our queue.') ) . "\r\n\r\n";
    $message .= $activate_url . "\r\n";

    wp_mail($user_email, sprintf(__('Please confirm your registration') ), $message);
    
    return false;
}
add_action( 'bp_core_signup_user', 'bpca_send_in_queue_email' );

/**
 * this adjusts the user status to reflect a 'waiting for activation' type of setting
 *
 * @since 0.1
 */
function bpca_deactivate_user($user_id) {
	global $bp, $wpdb;
	update_user_meta( $user_id, 'bpca_activated_by_userID', '0' );
}

add_action('bp_core_signup_user', 'bpca_deactivate_user');

/**
 * Removing the current BuddyPress activation because in reality they are activated, but still in the queue
 *
 * @since 0.1
 */
remove_action( 'wp', 'bp_core_screen_activation');

/**
 * This is the new activation function, which is a near duplicate to the one in bp-core-signup.php
 * The difference here is that we basically change a little text and redirect them to our own activation confirmation screen
 *
 * @since 0.1
 */
function bpca_core_screen_activation() {
	global $bp, $wpdb;

	if ( BP_ACTIVATION_SLUG != $bp->current_component )
		return false;

	/* Check if an activation key has been passed */
	if ( isset( $_GET['key'] ) ) {

		require_once( ABSPATH . WPINC . '/registration.php' );

		/* Activate the signup */
		$user = apply_filters( 'bp_core_activate_account', bp_core_activate_signup( $_GET['key'] ) );

		/* If there was errors, add a message and redirect */
		if ( $user->errors ) {
			bp_core_add_message( __( 'There was an error validating your account, please try again.', 'buddypress' ), 'error' );
			bp_core_redirect( $bp->root_domain . '/' . BP_ACTIVATION_SLUG );
		}

		/* Check for an uploaded avatar and move that to the correct user folder */
		if ( bp_core_is_multisite() )
			$hashed_key = wp_hash( $_GET['key'] );
		else
			$hashed_key = wp_hash( $user );

		/* Check if the avatar folder exists. If it does, move rename it, move it and delete the signup avatar dir */
		if ( file_exists( BP_AVATAR_UPLOAD_PATH . '/avatars/signups/' . $hashed_key ) )
			@rename( BP_AVATAR_UPLOAD_PATH . '/avatars/signups/' . $hashed_key, BP_AVATAR_UPLOAD_PATH . '/avatars/' . $user );

		bp_core_add_message( __( 'Your account is now confirmed, and awaiting for a current member to activate you!', 'buddypress' ) );

                // Update User Status to '3' which indiciates they are confirmed (and technicaly self-activated), but awaiting activation from someone in the community
                $result = $wpdb->query( $wpdb->prepare("UPDATE wp_users SET user_status = 3 WHERE ID = ".$user) );

		$bp->activation_complete = true;
	}


	bp_core_redirect( '/invites/activated' );
}
add_action( 'wp', 'bpca_core_screen_activation', 3 );

/**
 * Stop user accounts logging in that have not been activated (user_status = 2 or 3)
 *
 * @since 0.1
 */
function bpca_core_signup_disable_inactive( $auth_obj, $username ) {
	global $bp, $wpdb;

	if ( !$user_id = bp_core_get_userid( $username ) )
		return $auth_obj;

	$user_status = (int) $wpdb->get_var( $wpdb->prepare( "SELECT user_status FROM $wpdb->users WHERE ID = %d", $user_id ) );

	if ( 2 == $user_status )
		bp_core_redirect( $bp->root_domain );
	else if ( 3 == $user_status )
                bp_core_redirect( '/invites/activated' );
        else
		return $auth_obj;
}
add_filter( 'authenticate', 'bpca_core_signup_disable_inactive', 11, 2 );


?>