<?php
/**
 * This file is included when BuddyPress is active, and after the plugins_loaded action.
 *
 * @author David Bisset <dbisset@dimensionmedia.com>
 * @package Community Activate for BuddyPress
 * @subpackage loader
 * @version 0.1
 *
 */

/**
 * Constant for third-party plugins to check if Community Activate is active
 */
define ( 'CACTIVATE_IS_INSTALLED', 1 );

/**
 * Version number
 */
define ( 'CACTIVATE_VERSION', '0.1.0' );

// The classes file holds all database access classes and functions
require ( dirname( __FILE__ ) . '/bp-community-activate-classes.php' );

// The ajax file holds all functions used in AJAX queries
require ( dirname( __FILE__ ) . '/bp-community-activate-ajax.php' );

// The cssjs file sets up and enqueue all CSS and JS files
require ( dirname( __FILE__ ) . '/bp-community-activate-cssjs.php' );

// The templatetags file contains classes and functions designed for use in template files
require ( dirname( __FILE__ ) . '/bp-community-activate-templatetags.php' );

/**
 * Set up global variables
 *
 * @global object $bp BuddyPress global settings
 * @since 0.1
 */
function bpca_setup_globals() {
	global $bp;

	$bp->cactivate->id = 'cactivate';
	$bp->cactivate->slug = CACTIVATE_SLUG;

	$bp->active_components[$bp->cactivate->slug] = $bp->cactivate->id;
}
add_action( 'bp_setup_globals', 'bpca_setup_globals' );

/**
 * Registers as a root component (example.com/cactivate/)
 *
 * @since 0.1
 */
function bpca_setup_root_component() {
	bp_core_add_root_component( CACTIVATE_SLUG );
}
add_action( 'bp_setup_root_components', 'bpca_setup_root_component' );


/**
 * Adds invite list item to the site's main navigation
 *
 * @global object $bp BuddyPress global settings
 * @since 0.1
 */
function bpca_add_to_site_nav() {
	global $bp;
?>
	<li<?php if ( bp_is_page( $bp->cactivate->slug ) ) : ?> class="selected"<?php endif; ?>>
		<a href="<?php echo site_url() ?>/<?php echo $bp->cactivate->slug ?>/"><?php _e( 'Invites', 'bpca' ) ?></a>
	</li>
<?php
}
add_action( 'bp_nav_items', 'bpca_add_to_site_nav' );

function bpca_setup_nav_directory() {
    global $bp, $is_member_page;

	if ( $bp->current_component == $bp->cactivate->slug && !$bp->current_action && !$bp->current_item  ) {
                $bp->is_directory = true;

                do_action( 'bpca_setup_nav_directory' );
                bp_core_load_template( apply_filters( 'cactivate_template_directory', 'invites/index' ) );
                return;
        }
}
add_action( 'wp', 'bpca_setup_nav_directory', 2 );

function bpca_activated_page() {
    global $bp, $is_member_page;

//        echo "<BR><BR><BR><BR>";
        
//        print_r ($bp->current_component);

	if ( $bp->current_component == 'invites' && $bp->current_action == 'activated'  ) {
//                $bp->is_directory = true;

                do_action( 'bpca_activated_page' );
                bp_core_load_template( apply_filters( 'cactivate_template_directory', 'activation/index' ) );
                return;
        }
}
add_action( 'wp', 'bpca_activated_page', 2 );

/**
 * Adds 'activated by xxxx on xxxx' message on profile pages
 *
 * @global object $bp BuddyPress global settings
 * @since 0.1
 */
function bpca_display_activation_in_profile() {
	global $bp;
	
	$invite = new BPCA_Invite;

	$invite->activated_user_id = $bp->displayed_user->id;
	
	if ($invite->activated_user_id) { 
		$info = $invite->get_activation_information();
		$user_name = $info['user_who_activated'][0]->display_name;
		$user_id = $info['user_who_activated'][0]->meta_value;

                $bpca_options = get_option( 'bp_community_activate' );

                

		if ($info['user_who_activated'][0]->display_name && $info['activation_date'][0]->meta_value && $bpca_options['show_who_activated'] == "yes") {
			echo "<tr><td class=\"label\">Activated by</label></td><td class=\"data\"><a href=\"".bp_core_get_user_domain( $user_id, $user_name ) ."\">".$user_name."</a> on ".date('F jS, Y', strtotime($info['activation_date'][0]->meta_value))."</td></tr>";
		}
//		print_r ($info);
	} else {
		return false;	
	}

}
add_action ( 'bp_profile_field_item', 'bpca_display_activation_in_profile');

/**
 * Adds 'number of people user has invited: xxxx' message on profile pages
 *
 * @global object $bp BuddyPress global settings
 * @since 0.1
 */
function bpca_display_invites_in_profile() {
	global $bp;

	$invite = new BPCA_Invite;

	$invite->user_id = $bp->displayed_user->id;

	if ($invite->user_id) {
		$info = $invite->get_invite_information();
		$invite_number = $info['invites_used'][0]->meta_value;

                //show_count_on_user_screen

                $bpca_options = get_option( 'bp_community_activate' );

		if ($invite_number && $bpca_options['show_count_on_user_screen'] == "yes") {
			echo "<tr><td class=\"label\">Activated</label></td><td class=\"data\">".$invite_number." users</td></tr>";
		}
//		print_r ($info);
	} else {
		return false;
	}

}
add_action ( 'bp_profile_field_item', 'bpca_display_invites_in_profile');


/**
 * Load template filter; allows site/super admin to override cactivate' templates in
 * their active theme and replace the ones that are stored in the plugin directory.
 *
 * @global object $bp BuddyPress global settings
 * @param string $found_template Contains any matches found for the requested template(s)
 * @param array $templates Page template(s) we want to load
 * @return string Absolute directory path to the requested template
 * @since 0.1
 */
function bpca_load_template_filter( $found_template, $templates ) {
	global $bp;

	if ( $bp->current_component != $bp->cactivate->slug || $found_template )
		return $found_template;

	$filtered_templates = array();
	$filtered_templates = locate_template( $templates, false );

	if ( is_array( $filtered_templates ) && isset( $filtered_templates[0] ) && $filtered_templates[0] )
		return apply_filters( 'bpca_load_template_filter', $filtered_templates[0] );

	$filtered_templates = array();
	foreach ( (array) $templates as $template ) {
		$url = dirname( __FILE__ ) . '/templates/' . $template;

		if ( file_exists( $url ) )
			$filtered_templates[] = $url;
	}

	if ( !isset( $filtered_templates[0] ) )
		$filtered_templates[0] = '';

	return apply_filters( 'bpca_load_template_filter', $filtered_templates[0] );
}
add_filter( 'bp_located_template', 'bpca_load_template_filter', 10, 2 );

/**
 * Includes, into page output, a template file. To be used within a template included by bp_core_load_template().
 *
 * @global object $bp BuddyPress global settings
 * @param array $templates Requested templates, in priority order
 * @see bp_core_add_root_component()
 * @see bp_core_load_template()
 * @since 0.1
 */
function bpca_load_template( $templates ) {
	global $bp;

	$alternative_templates = array();

	foreach ( $templates as $template ) {
		$uri = explode( '/', $template );
		$uri_size = count( $uri );

		if ( !is_array( $uri ) || $uri_size < 2 )
			continue;

		if ( '/' == $uri[0] ) {
			unset( $uri[0] );
			$uri = array_merge( $uri, array() );  // Reset the keys by merging with an empty array
		}

		if ( !in_array( $uri[0], $bp->root_components ) || 'single' != $uri[1] )
			continue;

		if ( $bp->current_item )
			$template_slug = $bp->current_item;
		elseif ( BP_MEMBERS_SLUG == $uri[0] && isset( $bp->displayed_user ) )
			$template_slug = $bp->displayed_user->userdata->user_login;
		else
			continue;

		$extension = strpos( $uri[$uri_size - 1], '.php' );
		if ( false === $extension )
			continue;

		$template_slug = apply_filters( 'bpca_load_template_slug', $template_slug, $template, $templates );
		if ( $template_slug ) {
			// Reconstruct the URL with the filename suffix
			$uri[$uri_size - 1] = substr( $uri[$uri_size - 1], 0, $extension ) . "-{$template_slug}.php";
			$alternative_templates[] = implode( '/', $uri );
		}
	}

	$templates = array_merge( apply_filters( 'bpca_load_template_alternatives', $alternative_templates, $templates ), $templates );
	if ( $located_template = apply_filters( 'bp_located_template', locate_template( $templates, false ), $templates, $alternative_templates ) )
		load_template( apply_filters( 'bp_load_template', $located_template ) );
}


function bpca_screen_waiting_list() {
        do_action( 'bpca_screen_waiting_list' );
        bp_core_load_template( apply_filters( 'bpca_template_directory', 'index' ) );
}


/********************************************************************************
 * Action Functions
 *
 */

function bpca_action_activate_user() {
	global $bp;

	if ( $bp->current_component != $bp->cactivate->slug || $bp->current_action != 'activate-user' )
		return false;

	$potential_activated_user_id = $bp->action_variables[0];

	if ( !is_numeric( $potential_activated_user_id ) || !isset( $potential_activated_user_id ) )
		return false;

	if ( $potential_activated_user_id == $bp->loggedin_user->id )
		return false;

        $attempt = bpca_activate_user( $bp->loggedin_user->id, $potential_activated_user_id );

	if ( $attempt == "overlimit" ) {
		bp_core_add_message( __( 'You have exceeded your activation quota.', 'bpca' ), 'error' );
	} else if ( $attempt == false ) {
		bp_core_add_message( __( 'User could not be activated at this time.', 'bpca' ), 'error' );
        } else {
		bp_core_add_message( __( 'User activated', 'buddypress' ) );
	}

	bp_core_redirect( wp_get_referer() );

	return false;
}
add_action( 'init', 'bpca_action_activate_user' );


/********************************************************************************
 * Business Functions
 *
 */

function bpca_activate_user( $initiator_userid, $potential_activated_user_id ) {
	global $bp;


	$invite = new BPCA_Invite;
	
	$invite->initiator_user_id = $initiator_userid;
	$invite->potential_activated_user_id = $potential_activated_user_id;
	$invite->date_activated = bp_core_current_time();

	if ( $invite->activate_user() == "overkill" ) {
		do_action( 'bpca_user_not_activated', $invite->initiator_user_id, $invite->potential_activated_user_id );
		return "overkill";
	} else if ( $invite->activate_user() ) {
		do_action( 'bpca_user_activated', $invite->initiator_user_id, $invite->potential_activated_user_id );
		return true;
        } else {
		do_action( 'bpca_user_not_activated', $invite->initiator_user_id, $invite->potential_activated_user_id );
		return false;
	}


}

/**
 * bpca_record_activity()
 *
 * This records the activation activity in the activity stream, if that is enabled
 *
 */
function bpca_record_activity( $user_id, $content, $item_id, $component_action = 'activate_user' ) {
	global $bp;

	if ( !bp_is_active( 'activity' ) )
		return false;

	switch ( $component_action ) {
		default:
		case 'activate_user':
			$entry = array(
				'action' =>  sprintf( __( '%1$s activated %2$s', 'bpca' ), bp_core_get_userlink( $bp->loggedin_user->id ), bp_core_get_userlink( $user_id ) ),
				'component' => $bp->cactivate->id,
				'type' => $component_action,
				'primary_link' => bp_core_get_user_domain( $user_id ),
				'user_id' => $bp->loggedin_user->id,
				'item_id' => $user_id,
			);
		break;

		case 'got_activated':
			$entry = array(
				'action' =>  sprintf( __( '%1$s was activated by %2$s', 'bpca' ), bp_core_get_userlink( $user_id ), bp_core_get_userlink( $bp->loggedin_user->id ) ),
				'component' => $bp->cactivate->id,
				'type' => $component_action,
				'primary_link' => bp_core_get_user_domain( $user_id ),
				'user_id' => $user_id,
				'item_id' => $bp->loggedin_user->id,
			);
		break;
	}

	return bp_activity_add( apply_filters( 'bpca_record_activity', $entry, $user_id, $content, $item_id, $component_action ) );
}
?>