<?php
/**
 * Contains classes and functions designed for use in template files.
 *
 * @author David Bisset <dbisset@dimensionmedia.com>
 * @package Community Activate for BuddyPress
 * @subpackage templatetags
 * @version 0.1
 *
 */


/**
 * The template loop class for Invites
 *
 * @since 2.0
 * @package Invites 
 * @subpackage templatetags
 */
class BPCA_Invite_Template {
	
	/**
	 * The current position in the loop
	 *
	 * @access public
	 * @var integer
	 * @see BPCA_Invite_Template
	 * @since 2.0
	 */
	var $current_invite = -1;

	/**
	 * An array containing all the Invite objects, fetched from the database
	 *
	 * @access public
	 * @var array
	 * @see BPCA_Invite_Template
	 * @see BPCA_Invite::get()
	 * @since 2.0
	 */
	var $invites;

	/**
	 * The current Invite in the loop
	 *
	 * @access public
	 * @var BPCA_Invite
	 * @see BPCA_Invite_Template
	 * @since 2.0
	 */
	var $invite;

	/**
	 * The number of Invites to loop through (affected by pagination)
	 *
	 * @access public
	 * @var integer
	 * @see BPCA_Invite_Template
	 * @since 2.0
	*/
	var $invite_count;

	/**
	 * The total number of Invites to loop through (NOT affected by pagination)
	 *
	 * @access public
	 * @var integer
	 * @see BPCA_Invite_Template
	 * @since 2.0
	 */
	var $total_invite_count;

	/**
	 * Are we currently in the Invites loops?
	 *
	 * @access public
	 * @var bool
	 * @see BPCA_Invite_Template
	 * @since 2.0
	 */
	var $in_the_loop;

	/**
	 * What number page are we on for pagination?
	 *
	 * @access public
	 * @var integer
	 * @see BPCA_Invite_Template
	 * @since 2.0
	 */
	var $pag_page;

	/**
	 * How many Invites are we showing per-page for pagination?
	 *
	 * @access public
	 * @var integer
	 * @see BPCA_Invite_Template
	 * @since 2.0
	 */
	var $pag_num;

	/**
	 * Retrieve paginated links for Invite pages
	 *
	 * @access public
	 * @var array|string String of page links or array of page links
	 * @see BPCA_Invite_Template
	 * @see paginate_links()
	 * @since 2.0
	 */
	var $pag_links;

	/**
	 * Constructor
	 *
	 * The Invites template tag loop. It will retrieve results per the criteria passed from the database, apply pagination (if required)
	 * and populate the class variables required for the template tag loop to work in the page templates.
	 *
	 * @access public
	 * @global object $bp BuddyPress global settings
	 * @param integer $user_id = '0'. User ID
	 * @param string $type = 'all'. Either: all | single - Right now, just using 'all' since there really are no invite types
	 * @param integer $page = '1'. Which page of results are we on, for pagination? Also, $page = 1 without a per_page will result in no pagination being applied to the results
	 * @param integer $per_page = '20'. How many Invites per page, for pagination?
	 * @param integer $max = '0'. How many Invites to return as a maximum?
	 * @param string $slug = ''. The slug of an Invite. Only required when $type is 'single' - IT MIGHT NOT EVEN BE NEEDED
	 * @param string $action = ''. Only get Invites with this action. Only required when $type is 'active_by_action' and $user_id != 0 and $populate_extras is true - DELETE THIS
	 * @param string $search_terms = ''. Search query
	 * @param bool $populate_extras = true. If true, fetch additional information about when or if $user_id had unlocked each Invite - IT MIGHT NOT EVEN BE NEEDED
	 * @param bool $skip_detail_page_result = true. On a single Invite page, bpca_setup_nav() will setup the Invite template objects appropiately. Set this to skip that prefetched value.
	 * @see BPCA_Invite::get()
	 * @see bpca_setup_nav()
	 * @since 2.0
	 * @todo Wrap all these arguments into an array, $args. Change bools to strings to better describe what variable is used for.
	 * @uses BPCA_Invite_Template
	 */
	function BPCA_Invite_Template( $user_id, $type, $page, $per_page, $max, $slug, $action, $search_terms, $populate_extras, $skip_detail_page_result ) {
		global $bp;

		$this->pag_page = isset( $_REQUEST['xpage'] ) ? intval( $_REQUEST['xpage'] ) : $page;
		$this->pag_num = isset( $_GET['num'] ) ? intval( $_GET['num'] ) : $per_page;

		$args = array( 'limit' => $max, 'page' => $this->pag_page, 'per_page' => $this->pag_num, 'populate_extras' => $populate_extras, 'search_terms' => $search_terms, 'skip_detail_page_result' => $skip_detail_page_result, 'slug' => $slug, 'action' => $action, 'type' => $type, 'user_id' => $user_id );
		$this->invites = BPCA_Invite::get( $args );

		// Item Requests
		if ( !$max || $max >= (int)$this->invites['total'] )
			$this->total_invite_count = (int)$this->invites['total'];
		else
			$this->total_invite_count = (int)$max;

		$this->invites = $this->invites['invites'];

		if ( $max ) {
			if ( $max >= count( $this->invites ) )
				$this->invite_count = count( $this->invites );
			else
				$this->invite_count = (int)$max;
		} else {
			$this->invite_count = count( $this->invites );
		}

		if ( (int)$this->total_invite_count && (int)$this->pag_num ) {
			$this->pag_links = paginate_links( array(
				'base' => add_query_arg( array( 'xpage' => '%#%', 'num' => $this->pag_num, 's' => $search_terms ) ),
				'format' => '',
				'total' => ceil( (int)$this->total_invite_count / (int)$this->pag_num ),
				'current' => (int)$this->pag_page,
				'prev_text' => '&larr;',
				'next_text' => '&rarr;',
				'mid_size' => 1
			) );
		}
	}

	/**
	 * Returns true if we have Invites matching the retrieval critera
	 *
	 * @access public
	 * @since 2.0
	 * @return bool
	 */
	function has_invites() {
		if ( $this->invite_count )
			return true;

		return false;
	}

	/**
	 * Returns the next Invite object in the template tag loop
	 *
	 * @access public
	 * @since 2.0
	 * @uses BPCA_Invite
	 * @return BPCA_Invite
	 */
	function next_invite() {
		$this->current_invite++;
		$this->invite = $this->invites[$this->current_invite];

		return $this->invite;
	}

	/**
	 * Rewinds the template tag loop back to the beginning
	 *
	 * @access public
	 * @since 2.0
	 */
	function rewind_invites() {
		$this->current_invite = -1;
		if ( $this->invite_count > 0 ) {
			$this->invite = $this->invites[0];
		}
	}

	/**
	 * Returns true if there is another Invite to go through the template tag loop
	 *
	 * @access public
	 * @since 2.0
	 * @return bool
	 */
	function user_invites() {
		if ( $this->current_invite + 1 < $this->invite_count ) {
			return true;
		} elseif ( $this->current_invite + 1 == $this->invite_count ) {
			do_action( 'bpca_loop_end' );
			// Do some cleaning up after the loop
			$this->rewind_invites();
		}

		$this->in_the_loop = false;
		return false;
	}

	/**
	 * Sets the current Invite in the template tag loop
	 *
	 * @access public
	 * @global object $bp BuddyPress global settings
	 * @since 2.0
	 */
	function the_invite() {
		global $bp;

		$this->in_the_loop = true;
		$this->invite = $this->next_invite();

		if ( 0 == $this->current_invite ) // loop has just started
			do_action( 'bpca_loop_start' );
	}
}


/**
 * The main template tag function that starts it all. Take criteria, queries the databases and paginates it
 *
 * @since 2.0
 * @param array|string $args See BPCA_Invite_Template
 * @uses BPCA_Invite_Template
 * @global BPCA_Invite_Template $invites_template Invites template tag object
 * @global object $bp BuddyPress global settings
 * @return bool Did any Invites match critera?
 */
function bpca_has_invites( $args = '' ) {
	global $invites_template, $bp;

	// User filtering
	if ( !empty( $bp->displayed_user->id ) )
		$user_id = $bp->displayed_user->id;
	elseif ( !empty( $bp->loggedin_user->id ) )
		$user_id = $bp->loggedin_user->id;

	// Type
	if ( $bp->is_single_item ) {
		// This might be redundant.
		$type = 'single';
		$slug = $bp->invites->current_invite->slug;
	}

	if ( isset( $_REQUEST['s'] ) )
		$search_terms = $_REQUEST['s'];

	$defaults = array(
		'skip_detail_page_result' => true,
		'max' => 0,
		'page' => 1,
		'per_page' => 20,
		'populate_extras' => true,
		'search_terms' => '',  //Pass search terms to return only matching Invites
		'slug' => '',  //Pass an Invite slug to only return that Invites
		'type' => 'all',
		'user_id' => 0,
		'action' => ''
	);

	$r = wp_parse_args( $args, $defaults );
	extract( $r );
	


//	print_r ($r);

	$invites_template = new BPCA_Invite_Template( (int)$user_id, $type, (int)$page, (int)$per_page, (int)$max, $slug, $action, $search_terms, (bool)$populate_extras, (bool)$skip_detail_page_result );
	return apply_filters( 'bpca_has_invites', $invites_template->has_invites() );
}

/**
 * Sets the current Achievement in the template tag loop
 *
 * @since 2.0
 * @global DPA_Achievement_Template $invites_template Achievements template tag object
 */
function bpca_the_invite() {
	global $invites_template;

	return $invites_template->the_invite();
}

/**
 * Returns true if there is another Achievement to go through the template tag loop
 *
 * @since 2.0
 * @global DPA_Achievement_Template $invites_template Achievements template tag object
 * @return bool
 */
function bpca_invites() {
	global $invites_template;

	return $invites_template->user_invites();
}
	
/**
 * Pagination link text for template tag loop
 *
 * @global DPA_Invite_Template $invites_template Invites template tag object
 * @global object $bp BuddyPress global settings
 * @since 2.0
 */
function bpca_invites_pagination_count() {
	global $invites_template, $bp;

	$start_num = intval( ( $invites_template->pag_page - 1 ) * $invites_template->pag_num ) + 1;
	$from_num = bp_core_number_format( $start_num );
	$to_num = bp_core_number_format( ( $start_num + ( $invites_template->pag_num - 1 ) > $invites_template->total_invite_count ) ? $invites_template->total_invite_count : $start_num + ( $invites_template->pag_num - 1 ) );
	$total = bp_core_number_format( $invites_template->total_invite_count );

	$pagination = sprintf( __( 'Viewing Invites %1$s to %2$s (of %3$s Invites)', 'bpca' ), $from_num, $to_num, $total );

	if ( isset( $_REQUEST['search_terms'] ) && $_REQUEST['search_terms'] && 'false' != $_REQUEST['search_terms'] )
		$pagination .= sprintf( __( ' matching &ldquo;%s&rdquo;', 'bpca' ), apply_filters( 'bpca_invites_search_query', $_REQUEST['search_terms'] ) );

	echo $pagination;
	?> &nbsp; <span class="ajax-loader"></span><?php
}	
	
/**
 * Template tag version of bpca_get_invite_directory_class()
 *
 * @uses bpca_get_invite_directory_class()
 * @since 2.0
 */
function bpca_invite_directory_class() {
	echo bpca_get_invite_directory_class();
}
	/**
	 * Returns the CSS classes for each Achievement in the Directory
	 *
	 * @since 2.0
	 * @return string
	 */
	function bpca_get_invite_directory_class() {
		$class = esc_attr( "invite" );

		return apply_filters( 'bpca_get_invite_directory_class', $class );
	}
	
	
/*** Member Functions **/	

function bpca_member_permalink() {
	echo bpca_get_member_permalink();
}
	function bpca_get_member_permalink() {
		global $invites_template;

		return apply_filters( 'bpca_get_member_permalink', bp_core_get_user_domain( $invites_template->invite->user_id ) );
	}
	function bpca_member_link() { echo bp_get_member_permalink(); }
	function bpca_get_member_link() { return bp_get_member_permalink(); }
	


function bpca_member_name() {
	echo apply_filters( 'bpca_member_name', bpca_get_member_name() );
}
	function bpca_get_member_name() {
		global $invites_template;
		
//		print_r ($invites_template);

		if ( empty($invites_template->invite->fullname) )
			$invites_template->invite->fullname = $invites_template->invite->display_name;

		return apply_filters( 'bpca_get_member_name', $invites_template->invite->fullname );
	}
	add_filter( 'bpca_get_member_name', 'wp_filter_kses' );
	add_filter( 'bpca_get_member_name', 'stripslashes' );
	add_filter( 'bpca_get_member_name', 'strip_tags' );

function bpca_member_register_date() {
	echo apply_filters( 'bpca_member_register_date', bpca_get_member_register_date() );
}
	function bpca_get_member_register_date() {
		global $invites_template;
	
		return bp_core_time_since($invites_template->invite->user_registered);
	}
	
function bpca_member_avatar( $args = '' ) {
	echo apply_filters( 'bp_member_avatar', bpca_get_member_avatar( $args ) );
}
	function bpca_get_member_avatar( $args = '' ) {
		global $bp, $invites_template;
		
		$activation_key = get_usermeta($invites_template->invite->user_id, 'activation_key');
		
		$defaults = array(
			'type' => 'thumb',
			'width' => false,
			'height' => false,
			'class' => 'avatar',
			'object' => 'user',
			'id' => false,
			'avatar_dir' => 'avatars/signups/',
			'item_id' => $activation_key, // the folder is the activation key
			'alt' => __( 'Member avatar', 'buddypress' )
		);

		$r = wp_parse_args( $args, $defaults );
		extract( $r, EXTR_SKIP );

		return apply_filters( 'bp_get_member_avatar', bp_core_fetch_avatar( array( 'item_id' => $invites_template->invite->user_id, 'type' => $type, 'alt' => $alt, 'css_id' => $id, 'class' => $class, 'width' => $width, 'height' => $height, 'avatar_dir' => $avatar_dir, 'item_id' => $item_id, 'type' => $type, 'email' => $invites_template->invite->user_email ) ) );
	}
	
function bpca_member_add_invite_button() {
	global $invites_template;

	$invite_status = ''; // dummy variable for now

/*	if ( null === $members_template->member->is_friend )
		$friend_status = 'not_friends';
	else
		$friend_status = ( 0 == $members_template->member->is_friend ) ? 'pending' : 'is_friend'; */

	echo bpca_get_add_invite_button( $invites_template->invite->user_id, $invite_status );
}
add_action( 'bpca_directory_invites_actions', 'bpca_member_add_invite_button' );

function bpca_add_invite_button( $potential_invited_id = 0, $friend_status = '' ) {
	echo bpca_get_add_invite_button( $potential_invited_id, $invite_status );
}
	function bpca_get_add_invite_button( $potential_invited_id = 0, $invite_status = false ) {
		global $bp, $invites_template;
				
		$button = array(
			'id'                => 'not_invited',
			'component'         => 'cactivates',
			'must_be_logged_in' => true,
			'block_self'        => true,
			'wrapper_class'     => 'invite-button not_invited',
			'wrapper_id'        => 'invite-button-' . $potential_invited_id,
			'link_class'        => '',
			'link_href'         => wp_nonce_url( $bp->root_domain . '/' . CACTIVATE_SLUG . '/activate-user/' . $potential_invited_id . '/', 'cactivate_invite_user' ),
			'link_text'         => __( 'Activate User', 'bpca' ),
			'link_title'        => __( 'Activate User', 'bpca' ),
			'link_id'           => 'invitee-' . $potential_invited_id,
			'link_rel'          => 'add',
			'link_class'        => 'add'
		);

/*		$is_friend = bp_is_friend( $potential_friend_id );

		if ( empty( $is_friend ) )
			return false;

		switch ( $is_friend ) {
			case 'pending' :
				$button = array(
					'id'                => 'pending',
					'component'         => 'friends',
					'must_be_logged_in' => true,
					'block_self'        => true,
					'wrapper_class'     => 'friendship-button pending',
					'wrapper_id'        => 'friendship-button-' . $potential_friend_id,
					'link_class'        => 'requested',
					'link_href'         => trailingslashit( $bp->loggedin_user->domain . $bp->friends->slug ),
					'link_text'         => __( 'Friendship Requested', 'buddypress' ),
					'link_title'        => __( 'Friendship Requested', 'buddypress' )
				);
				break;

			case 'is_friend' :
				$button = array(
					'id'                => 'is_friend',
					'component'         => 'friends',
					'must_be_logged_in' => true,
					'block_self'        => true,
					'wrapper_class'     => 'friendship-button is_friend',
					'wrapper_id'        => 'friendship-button-' . $potential_friend_id,
					'link_class'        => '',
					'link_href'         => wp_nonce_url( $bp->loggedin_user->domain . $bp->friends->slug . '/remove-friend/' . $potential_friend_id . '/', 'friends_remove_friend' ),
					'link_text'         => __( 'Cancel Friendship', 'buddypress' ),
					'link_title'        => __( 'Cancel Friendship', 'buddypress' ),
					'link_id'           => 'friend-' . $potential_friend_id,
					'link_rel'          => 'remove',
					'link_class'        => 'remove'
				);
				break;

			default:
				$button = array(
					'id'                => 'not_friends',
					'component'         => 'friends',
					'must_be_logged_in' => true,
					'block_self'        => true,
					'wrapper_class'     => 'friendship-button not_friends',
					'wrapper_id'        => 'friendship-button-' . $potential_friend_id,
					'link_class'        => '',
					'link_href'         => wp_nonce_url( $bp->loggedin_user->domain . $bp->friends->slug . '/add-friend/' . $potential_friend_id . '/', 'friends_add_friend' ),
					'link_text'         => __( 'Add Friend', 'buddypress' ),
					'link_title'        => __( 'Add Friend', 'buddypress' ),
					'link_id'           => 'friend-' . $potential_friend_id,
					'link_rel'          => 'add',
					'link_class'        => 'add'
				);
				break;
		}*/

		// Filter and return the HTML button
		return bp_get_button( apply_filters( 'bp_get_add_friend_button', $button ) );
	}
	
/**
 * Search form template tag for the Directory
 *
 * @since 2.0
 */
function bpca_directory_invites_search_form() {
	$search_value = '';
	if ( !empty( $_REQUEST['s'] ) )
	 	$search_value = $_REQUEST['s'];
?>
	<form action="" method="get" id="search-invites-form">
		<label><input type="text" name="s" placeholder="<?php echo esc_attr( __( 'Search Invites', 'bpca' ) ) ?>" id="invites_search" value="<?php echo esc_attr( $search_value ) ?>" /></label>
		<input type="submit" id="invites_search_submit" name="invites_search_submit" value="<?php _e( 'Search', 'bpca' ) ?>" />
	</form>
<?php
}
?>